@@ -77,7 +77,7 @@
  * object paths that you wish to register are registered before
  * #GApplication attempts to acquire the bus name of your application
  * (which happens in g_application_register()).  Unfortunately, this
- * means that you can not use g_application_get_is_remote() to decide if
+ * means that you cannot use g_application_get_is_remote() to decide if
  * you want to register object paths.
  *
  * GApplication provides convenient life cycle management by maintaining
@@ -945,7 +945,7 @@ g_application_get_is_registered (GApplication *application)
  * perform actions on @application will result in the actions being
  * performed by the primary instance.
  *
- * The value of this property can not be accessed before
+ * The value of this property cannot be accessed before
  * g_application_register() has been called.  See
  * g_application_get_is_registered().
  *
@@ -1006,14 +1006,17 @@ g_application_register (GApplication  *application,
 
   if (!application->priv->is_registered)
     {
-      application->priv->impl =
-        g_application_impl_register (application, application->priv->id,
-                                     application->priv->flags,
-                                     &application->priv->remote_actions,
-                                     cancellable, error);
+      if (~application->priv->flags & G_APPLICATION_NON_UNIQUE)
+        {
+          application->priv->impl =
+            g_application_impl_register (application, application->priv->id,
+                                         application->priv->flags,
+                                         &application->priv->remote_actions,
+                                         cancellable, error);
 
-      if (application->priv->impl == NULL)
-        return FALSE;
+          if (application->priv->impl == NULL)
+            return FALSE;
+        }
 
       application->priv->is_remote = application->priv->remote_actions != NULL;
       application->priv->is_registered = TRUE;
@@ -1171,7 +1174,7 @@ g_application_open (GApplication  *application,
  * g_application_run:
  * @application: a #GApplication
  * @argc: the argc from main() (or 0 if @argv is %NULL)
- * @argv: (array length=argc): the argv from main(), or %NULL
+ * @argv: (array length=argc) (allow-none): the argv from main(), or %NULL
  * @returns: the exit status
  *
  * Runs the application.
@@ -1326,6 +1329,8 @@ g_application_run (GApplication  *application,
   if (application->priv->impl)
     g_application_impl_flush (application->priv->impl);
 
+  g_settings_sync ();
+
   return status;
 }
 
