@@ -1525,6 +1525,8 @@ parse_value_from_blob (GMemoryInputStream    *mis,
           g_free (s);
         }
     }
+#else
+  is_leaf = is_leaf; /* To avoid -Wunused-but-set-variable */
 #endif /* DEBUG_SERIALIZER */
 
   /* sink the reference */
@@ -2151,12 +2153,9 @@ append_body_to_blob (GVariant             *value,
                      GDataOutputStream    *dos,
                      GError              **error)
 {
-  gboolean ret;
   GVariant *item;
   GVariantIter iter;
 
-  ret = FALSE;
-
   if (!g_variant_is_of_type (value, G_VARIANT_TYPE_TUPLE))
     {
       g_set_error (error,
@@ -3198,8 +3197,8 @@ g_dbus_message_lock (GDBusMessage *message)
  * This operation can fail if e.g. @message contains file descriptors
  * and the per-process or system-wide open files limit is reached.
  *
- * Returns: (transfer full): A new #GDBusMessage or %NULL if @error is set.
- *     Free with g_object_unref().
+ * Returns: (transfer full): A new #GDBusMessage or %NULL if @error is set. Free with
+ * g_object_unref().
  *
  * Since: 2.26
  */
@@ -3213,7 +3212,7 @@ g_dbus_message_copy (GDBusMessage  *message,
   GVariant *header_value;
 
   g_return_val_if_fail (G_IS_DBUS_MESSAGE (message), NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   ret = g_dbus_message_new ();
   ret->type                   = message->type;
@@ -3256,3 +3255,4 @@ g_dbus_message_copy (GDBusMessage  *message,
  out:
   return ret;
 }
+
