@@ -93,7 +93,6 @@
 #include <gio/gnetworkaddress.h>
 #include <gio/gnetworkservice.h>
 #include <gio/goutputstream.h>
-#include <gio/gperiodic.h>
 #include <gio/gpermission.h>
 #include <gio/gpollableinputstream.h>
 #include <gio/gpollableoutputstream.h>
@@ -120,6 +119,7 @@
 #include <gio/gtcpwrapperconnection.h>
 #include <gio/gthemedicon.h>
 #include <gio/gthreadedsocketservice.h>
+#include <gio/gtimezonemonitor.h>
 #include <gio/gtlsbackend.h>
 #include <gio/gtlscertificate.h>
 #include <gio/gtlsclientconnection.h>
