@@ -57,11 +57,11 @@
 #include <string.h>
 
 #include "garray.h"
-#include "gbitlock.h"
 #include "gslist.h"
 #include "gtestutils.h"
 #include "gtimer.h"
 
+
 /**
  * SECTION:threads
  * @title: Threads
@@ -83,14 +83,11 @@
  * portable means for writing multi-threaded software. There are
  * primitives for mutexes to protect the access to portions of memory
  * (#GMutex, #GStaticMutex, #G_LOCK_DEFINE, #GStaticRecMutex and
- * #GStaticRWLock). There is a facility to use individual bits for
- * locks (g_bit_lock()). There are primitives for condition variables to
+ * #GStaticRWLock). There are primitives for condition variables to
  * allow synchronization of threads (#GCond).  There are primitives for
  * thread-private data - data that every thread has a private instance
- * of (#GPrivate, #GStaticPrivate). There are facilities for one-time
- * initialization (#GOnce, g_once_init_enter()). Last but definitely
- * not least there are primitives to portably create and manage
- * threads (#GThread).
+ * of (#GPrivate, #GStaticPrivate). Last but definitely not least there
+ * are primitives to portably create and manage threads (#GThread).
  *
  * The threading system is initialized with g_thread_init(), which
  * takes an optional custom thread implementation or %NULL for the
@@ -258,22 +255,17 @@ g_thread_error_quark (void)
 }
 
 /* Miscellaneous Structures {{{1 ------------------------------------------ */
+/* Keep this in sync with GRealThread in gmain.c! */
 typedef struct _GRealThread GRealThread;
 struct  _GRealThread
 {
   GThread thread;
-  /* Bit 0 protects private_data. To avoid deadlocks, do not block while
-   * holding this (particularly on the g_thread lock). */
-  volatile gint private_data_lock;
-  GArray *private_data;
+  gpointer private_data;
   GRealThread *next;
   gpointer retval;
   GSystemThread system_thread;
 };
 
-#define LOCK_PRIVATE_DATA(self)   g_bit_lock (&(self)->private_data_lock, 0)
-#define UNLOCK_PRIVATE_DATA(self) g_bit_unlock (&(self)->private_data_lock, 0)
-
 typedef struct _GStaticPrivateNode GStaticPrivateNode;
 struct _GStaticPrivateNode
 {
@@ -451,7 +443,7 @@ GThreadFunctions g_thread_functions_for_glib_use = {
  * #GStaticMutex provides a simpler and safer way of doing this.
  *
  * If you want to use a mutex, and your code should also work without
- * calling g_thread_init() first, then you can not use a #GMutex, as
+ * calling g_thread_init() first, then you cannot use a #GMutex, as
  * g_mutex_new() requires that the thread system be initialized. Use a
  * #GStaticMutex instead.
  *
@@ -744,7 +736,7 @@ GThreadFunctions g_thread_functions_for_glib_use = {
  * <note><para>@destructor is used quite differently from @notify in
  * g_static_private_set().</para></note>
  *
- * <note><para>A #GPrivate can not be freed. Reuse it instead, if you
+ * <note><para>A #GPrivate cannot be freed. Reuse it instead, if you
  * can, to avoid shortage, or use #GStaticPrivate.</para></note>
  *
  * <note><para>This function will abort if g_thread_init() has not been
@@ -871,7 +863,7 @@ static GMutex   *g_once_mutex = NULL;
 static GCond    *g_once_cond = NULL;
 static GPrivate *g_thread_specific_private = NULL;
 static GRealThread *g_thread_all_threads = NULL;
-static GSList   *g_thread_free_indices = NULL;
+static GSList   *g_thread_free_indeces = NULL;
 static GSList*   g_once_init_list = NULL;
 
 G_LOCK_DEFINE_STATIC (g_thread);
@@ -1268,30 +1260,19 @@ g_static_mutex_init (GStaticMutex *mutex)
 GMutex *
 g_static_mutex_get_mutex_impl (GMutex** mutex)
 {
-  GMutex *result;
-
   if (!g_thread_supported ())
     return NULL;
 
-  result = g_atomic_pointer_get (mutex);
+  g_assert (g_once_mutex);
 
-  if (!result)
-    {
-      g_assert (g_once_mutex);
+  g_mutex_lock (g_once_mutex);
 
-      g_mutex_lock (g_once_mutex);
+  if (!(*mutex))
+    g_atomic_pointer_set (mutex, g_mutex_new());
 
-      result = *mutex;
-      if (!result)
-        {
-          result = g_mutex_new ();
-          g_atomic_pointer_set (mutex, result);
-        }
+  g_mutex_unlock (g_once_mutex);
 
-      g_mutex_unlock (g_once_mutex);
-    }
-
-  return result;
+  return *mutex;
 }
 
 /* IMPLEMENTATION NOTE:
@@ -1670,18 +1651,18 @@ g_static_private_get (GStaticPrivate *private_key)
 {
   GRealThread *self = (GRealThread*) g_thread_self ();
   GArray *array;
-  gpointer ret = NULL;
-
-  LOCK_PRIVATE_DATA (self);
 
   array = self->private_data;
+  if (!array)
+    return NULL;
 
-  if (array && private_key->index != 0 && private_key->index <= array->len)
-    ret = g_array_index (array, GStaticPrivateNode,
-                         private_key->index - 1).data;
-
-  UNLOCK_PRIVATE_DATA (self);
-  return ret;
+  if (!private_key->index)
+    return NULL;
+  else if (private_key->index <= array->len)
+    return g_array_index (array, GStaticPrivateNode,
+			  private_key->index - 1).data;
+  else
+    return NULL;
 }
 
 /**
@@ -1713,31 +1694,6 @@ g_static_private_set (GStaticPrivate *private_key,
   GArray *array;
   static guint next_index = 0;
   GStaticPrivateNode *node;
-  gpointer ddata = NULL;
-  GDestroyNotify ddestroy = NULL;
-
-  if (!private_key->index)
-    {
-      G_LOCK (g_thread);
-
-      if (!private_key->index)
-	{
-	  if (g_thread_free_indices)
-	    {
-	      private_key->index =
-		GPOINTER_TO_UINT (g_thread_free_indices->data);
-	      g_thread_free_indices =
-		g_slist_delete_link (g_thread_free_indices,
-				     g_thread_free_indices);
-	    }
-	  else
-	    private_key->index = ++next_index;
-	}
-
-      G_UNLOCK (g_thread);
-    }
-
-  LOCK_PRIVATE_DATA (self);
 
   array = self->private_data;
   if (!array)
@@ -1746,21 +1702,46 @@ g_static_private_set (GStaticPrivate *private_key,
       self->private_data = array;
     }
 
+  if (!private_key->index)
+    {
+      G_LOCK (g_thread);
+
+      if (!private_key->index)
+	{
+	  if (g_thread_free_indeces)
+	    {
+	      private_key->index =
+		GPOINTER_TO_UINT (g_thread_free_indeces->data);
+	      g_thread_free_indeces =
+		g_slist_delete_link (g_thread_free_indeces,
+				     g_thread_free_indeces);
+	    }
+	  else
+	    private_key->index = ++next_index;
+	}
+
+      G_UNLOCK (g_thread);
+    }
+
   if (private_key->index > array->len)
     g_array_set_size (array, private_key->index);
 
   node = &g_array_index (array, GStaticPrivateNode, private_key->index - 1);
+  if (node->destroy)
+    {
+      gpointer ddata = node->data;
+      GDestroyNotify ddestroy = node->destroy;
 
-  ddata = node->data;
-  ddestroy = node->destroy;
+      node->data = data;
+      node->destroy = notify;
 
-  node->data = data;
-  node->destroy = notify;
-
-  UNLOCK_PRIVATE_DATA (self);
-
-  if (ddestroy)
-    ddestroy (ddata);
+      ddestroy (ddata);
+    }
+  else
+    {
+      node->data = data;
+      node->destroy = notify;
+    }
 }
 
 /**
@@ -1778,8 +1759,7 @@ void
 g_static_private_free (GStaticPrivate *private_key)
 {
   guint idx = private_key->index;
-  GRealThread *thread, *next;
-  GArray *garbage = NULL;
+  GRealThread *thread;
 
   if (!idx)
     return;
@@ -1789,16 +1769,10 @@ g_static_private_free (GStaticPrivate *private_key)
   G_LOCK (g_thread);
 
   thread = g_thread_all_threads;
-
-  for (thread = g_thread_all_threads; thread; thread = next)
+  while (thread)
     {
-      GArray *array;
-
-      next = thread->next;
-
-      LOCK_PRIVATE_DATA (thread);
-
-      array = thread->private_data;
+      GArray *array = thread->private_data;
+      thread = thread->next;
 
       if (array && idx <= array->len)
 	{
@@ -1813,41 +1787,15 @@ g_static_private_free (GStaticPrivate *private_key)
 
           if (ddestroy)
             {
-              /* defer non-trivial destruction til after we've finished
-               * iterating, since we must continue to hold the lock */
-              if (garbage == NULL)
-                garbage = g_array_new (FALSE, TRUE,
-                                       sizeof (GStaticPrivateNode));
-
-              g_array_set_size (garbage, garbage->len + 1);
-
-              node = &g_array_index (garbage, GStaticPrivateNode,
-                                     garbage->len - 1);
-              node->data = ddata;
-              node->destroy = ddestroy;
+              G_UNLOCK (g_thread);
+              ddestroy (ddata);
+              G_LOCK (g_thread);
             }
 	}
-
-      UNLOCK_PRIVATE_DATA (thread);
     }
-  g_thread_free_indices = g_slist_prepend (g_thread_free_indices,
+  g_thread_free_indeces = g_slist_prepend (g_thread_free_indeces,
 					   GUINT_TO_POINTER (idx));
   G_UNLOCK (g_thread);
-
-  if (garbage)
-    {
-      guint i;
-
-      for (i = 0; i < garbage->len; i++)
-        {
-          GStaticPrivateNode *node;
-
-          node = &g_array_index (garbage, GStaticPrivateNode, i);
-          node->destroy (node->data);
-        }
-
-      g_array_free (garbage, TRUE);
-    }
 }
 
 /* GThread Extra Functions {{{1 ------------------------------------------- */
@@ -1857,15 +1805,9 @@ g_thread_cleanup (gpointer data)
   if (data)
     {
       GRealThread* thread = data;
-      GArray *array;
-
-      LOCK_PRIVATE_DATA (thread);
-      array = thread->private_data;
-      thread->private_data = NULL;
-      UNLOCK_PRIVATE_DATA (thread);
-
-      if (array)
+      if (thread->private_data)
 	{
+	  GArray* array = thread->private_data;
 	  guint i;
 
 	  for (i = 0; i < array->len; i++ )
