@@ -129,28 +129,12 @@ struct _GTimeZone
   gint     ref_count;
 };
 
+G_LOCK_DEFINE_STATIC (local_timezone);
+static GTimeZone *local_timezone;
+
 G_LOCK_DEFINE_STATIC (time_zones);
 static GHashTable/*<string?, GTimeZone>*/ *time_zones;
 
-static guint
-g_str_hash0 (gconstpointer data)
-{
-  return data ? g_str_hash (data) : 0;
-}
-
-static gboolean
-g_str_equal0 (gconstpointer a,
-              gconstpointer b)
-{
-  if (a == b)
-    return TRUE;
-
-  if (!a || !b)
-    return FALSE;
-
-  return g_str_equal (a, b);
-}
-
 /**
  * g_time_zone_unref:
  * @tz: a #GTimeZone
@@ -166,9 +150,28 @@ g_time_zone_unref (GTimeZone *tz)
 
   if (g_atomic_int_dec_and_test (&tz->ref_count))
     {
-      G_LOCK(time_zones);
-      g_hash_table_remove (time_zones, tz->name);
-      G_UNLOCK(time_zones);
+      if G_UNLIKELY (tz == local_timezone)
+        {
+          g_critical ("The last reference on the local timezone was just "
+                      "dropped, but GTimeZone itself still owns one.  This "
+                      "means that g_time_zone_unref() was called too many "
+                      "times.  Restoring the refcount to 1.");
+
+          /* We don't want to just inc this back again since if there
+           * are refcounting bugs in the code then maybe we are already
+           * at -1 and inc will just take us back to 0.  Set to 1 to be
+           * sure.
+           */
+          tz->ref_count = 1;
+          return;
+        }
+
+      if (tz->name != NULL)
+        {
+          G_LOCK(time_zones);
+          g_hash_table_remove (time_zones, tz->name);
+          G_UNLOCK(time_zones);
+        }
 
       if (tz->zoneinfo)
         g_buffer_unref (tz->zoneinfo);
@@ -205,43 +208,43 @@ g_time_zone_ref (GTimeZone *tz)
  *  - hh is 00 to 23
  *  - mm is 00 to 59
  */
-gboolean
-parse_time (const gchar *time,
+static gboolean
+parse_time (const gchar *time_,
             gint32      *offset)
 {
-  if (*time < '0' || '2' < *time)
+  if (*time_ < '0' || '2' < *time_)
     return FALSE;
 
-  *offset = 10 * 60 * 60 * (*time++ - '0');
+  *offset = 10 * 60 * 60 * (*time_++ - '0');
 
-  if (*time < '0' || '9' < *time)
+  if (*time_ < '0' || '9' < *time_)
     return FALSE;
 
-  *offset += 60 * 60 * (*time++ - '0');
+  *offset += 60 * 60 * (*time_++ - '0');
 
   if (*offset > 23 * 60 * 60)
     return FALSE;
 
-  if (*time == '\0')
+  if (*time_ == '\0')
     return TRUE;
 
-  if (*time == ':')
-    time++;
+  if (*time_ == ':')
+    time_++;
 
-  if (*time < '0' || '5' < *time)
+  if (*time_ < '0' || '5' < *time_)
     return FALSE;
 
-  *offset += 10 * 60 * (*time++ - '0');
+  *offset += 10 * 60 * (*time_++ - '0');
 
-  if (*time < '0' || '9' < *time)
+  if (*time_ < '0' || '9' < *time_)
     return FALSE;
 
-  *offset += 60 * (*time++ - '0');
+  *offset += 60 * (*time_++ - '0');
 
-  return *time == '\0';
+  return *time_ == '\0';
 }
 
-gboolean
+static gboolean
 parse_constant_offset (const gchar *name,
                        gint32      *offset)
 {
@@ -345,10 +348,13 @@ g_time_zone_new (const gchar *identifier)
 
   G_LOCK (time_zones);
   if (time_zones == NULL)
-    time_zones = g_hash_table_new (g_str_hash0,
-                                   g_str_equal0);
+    time_zones = g_hash_table_new (g_str_hash, g_str_equal);
+
+  if (identifier)
+    tz = g_hash_table_lookup (time_zones, identifier);
+  else
+    tz = NULL;
 
-  tz = g_hash_table_lookup (time_zones, identifier);
   if (tz == NULL)
     {
       tz = g_slice_new0 (GTimeZone);
@@ -412,7 +418,8 @@ g_time_zone_new (const gchar *identifier)
             }
         }
 
-      g_hash_table_insert (time_zones, tz->name, tz);
+      if (identifier)
+        g_hash_table_insert (time_zones, tz->name, tz);
     }
   g_atomic_int_inc (&tz->ref_count);
   G_UNLOCK (time_zones);
@@ -461,7 +468,46 @@ g_time_zone_new_utc (void)
 GTimeZone *
 g_time_zone_new_local (void)
 {
-  return g_time_zone_new (getenv ("TZ"));
+  GTimeZone *result;
+
+  G_LOCK (local_timezone);
+  if (local_timezone == NULL)
+    local_timezone = g_time_zone_new (getenv ("TZ"));
+
+  result = g_time_zone_ref (local_timezone);
+  G_UNLOCK (local_timezone);
+
+  return result;
+}
+
+/**
+ * g_time_zone_refresh_local:
+ *
+ * Notifies #GTimeZone that the local timezone may have changed.
+ *
+ * In response, #GTimeZone will drop its cache of the local time zone.
+ * No existing #GTimeZone will be modified and no #GDateTime will change
+ * its timezone but future calls to g_time_zone_new_local() will start
+ * returning the new timezone.
+ *
+ * #GTimeZone does no monitoring of the local timezone on its own, which
+ * is why you have to call this function to notify it of the change.
+ *
+ * If you use #GTimeZoneMonitor to watch for changes then this function
+ * will automatically be called for you.
+ **/
+void
+g_time_zone_refresh_local (void)
+{
+  GTimeZone *drop_this_ref = NULL;
+
+  G_LOCK (local_timezone);
+  drop_this_ref = local_timezone;
+  local_timezone = NULL;
+  G_UNLOCK (local_timezone);
+
+  if (drop_this_ref)
+    g_time_zone_unref (drop_this_ref);
 }
 
 /* Internal helpers {{{1 */
@@ -548,34 +594,34 @@ interval_valid (GTimeZone *tz,
 /**
  * g_time_zone_adjust_time:
  * @tz: a #GTimeZone
- * @type: the #GTimeType of @time
- * @time: a pointer to a number of seconds since January 1, 1970
+ * @type: the #GTimeType of @time_
+ * @time_: a pointer to a number of seconds since January 1, 1970
  *
- * Finds an interval within @tz that corresponds to the given @time,
- * possibly adjusting @time if required to fit into an interval.
- * The meaning of @time depends on @type.
+ * Finds an interval within @tz that corresponds to the given @time_,
+ * possibly adjusting @time_ if required to fit into an interval.
+ * The meaning of @time_ depends on @type.
  *
  * This function is similar to g_time_zone_find_interval(), with the
  * difference that it always succeeds (by making the adjustments
  * described below).
  *
  * In any of the cases where g_time_zone_find_interval() succeeds then
- * this function returns the same value, without modifying @time.
+ * this function returns the same value, without modifying @time_.
  *
- * This function may, however, modify @time in order to deal with
- * non-existent times.  If the non-existent local @time of 02:30 were
+ * This function may, however, modify @time_ in order to deal with
+ * non-existent times.  If the non-existent local @time_ of 02:30 were
  * requested on March 13th 2010 in Toronto then this function would
- * adjust @time to be 03:00 and return the interval containing the
+ * adjust @time_ to be 03:00 and return the interval containing the
  * adjusted time.
  *
- * Returns: the interval containing @time, never -1
+ * Returns: the interval containing @time_, never -1
  *
  * Since: 2.26
  **/
 gint
 g_time_zone_adjust_time (GTimeZone *tz,
                          GTimeType  type,
-                         gint64    *time)
+                         gint64    *time_)
 {
   gint i;
 
@@ -585,47 +631,47 @@ g_time_zone_adjust_time (GTimeZone *tz,
   /* find the interval containing *time UTC
    * TODO: this could be binary searched (or better) */
   for (i = 0; i < tz->timecnt; i++)
-    if (*time <= interval_end (tz, i))
+    if (*time_ <= interval_end (tz, i))
       break;
 
-  g_assert (interval_start (tz, i) <= *time && *time <= interval_end (tz, i));
+  g_assert (interval_start (tz, i) <= *time_ && *time_ <= interval_end (tz, i));
 
   if (type != G_TIME_TYPE_UNIVERSAL)
     {
-      if (*time < interval_local_start (tz, i))
+      if (*time_ < interval_local_start (tz, i))
         /* if time came before the start of this interval... */
         {
           i--;
 
           /* if it's not in the previous interval... */
-          if (*time > interval_local_end (tz, i))
+          if (*time_ > interval_local_end (tz, i))
             {
               /* it doesn't exist.  fast-forward it. */
               i++;
-              *time = interval_local_start (tz, i);
+              *time_ = interval_local_start (tz, i);
             }
         }
 
-      else if (*time > interval_local_end (tz, i))
+      else if (*time_ > interval_local_end (tz, i))
         /* if time came after the end of this interval... */
         {
           i++;
 
           /* if it's not in the next interval... */
-          if (*time < interval_local_start (tz, i))
+          if (*time_ < interval_local_start (tz, i))
             /* it doesn't exist.  fast-forward it. */
-            *time = interval_local_start (tz, i);
+            *time_ = interval_local_start (tz, i);
         }
 
       else if (interval_isdst (tz, i) != type)
         /* it's in this interval, but dst flag doesn't match.
          * check neighbours for a better fit. */
         {
-          if (i && *time <= interval_local_end (tz, i - 1))
+          if (i && *time_ <= interval_local_end (tz, i - 1))
             i--;
 
           else if (i < tz->timecnt &&
-                   *time >= interval_local_start (tz, i + 1))
+                   *time_ >= interval_local_start (tz, i + 1))
             i++;
         }
     }
@@ -636,18 +682,18 @@ g_time_zone_adjust_time (GTimeZone *tz,
 /**
  * g_time_zone_find_interval:
  * @tz: a #GTimeZone
- * @type: the #GTimeType of @time
- * @time: a number of seconds since January 1, 1970
+ * @type: the #GTimeType of @time_
+ * @time_: a number of seconds since January 1, 1970
  *
- * Finds an the interval within @tz that corresponds to the given @time.
- * The meaning of @time depends on @type.
+ * Finds an the interval within @tz that corresponds to the given @time_.
+ * The meaning of @time_ depends on @type.
  *
  * If @type is %G_TIME_TYPE_UNIVERSAL then this function will always
  * succeed (since universal time is monotonic and continuous).
  *
- * Otherwise @time is treated is local time.  The distinction between
+ * Otherwise @time_ is treated is local time.  The distinction between
  * %G_TIME_TYPE_STANDARD and %G_TIME_TYPE_DAYLIGHT is ignored except in
- * the case that the given @time is ambiguous.  In Toronto, for example,
+ * the case that the given @time_ is ambiguous.  In Toronto, for example,
  * 01:30 on November 7th 2010 occured twice (once inside of daylight
  * savings time and the next, an hour later, outside of daylight savings
  * time).  In this case, the different value of @type would result in a
@@ -658,14 +704,14 @@ g_time_zone_adjust_time (GTimeZone *tz,
  * forward to begin daylight savings time).  -1 is returned in that
  * case.
  *
- * Returns: the interval containing @time, or -1 in case of failure
+ * Returns: the interval containing @time_, or -1 in case of failure
  *
  * Since: 2.26
  */
 gint
 g_time_zone_find_interval (GTimeZone *tz,
                            GTimeType  type,
-                           gint64     time)
+                           gint64     time_)
 {
   gint i;
 
@@ -673,30 +719,30 @@ g_time_zone_find_interval (GTimeZone *tz,
     return 0;
 
   for (i = 0; i < tz->timecnt; i++)
-    if (time <= interval_end (tz, i))
+    if (time_ <= interval_end (tz, i))
       break;
 
   if (type == G_TIME_TYPE_UNIVERSAL)
     return i;
 
-  if (time < interval_local_start (tz, i))
+  if (time_ < interval_local_start (tz, i))
     {
-      if (time > interval_local_end (tz, --i))
+      if (time_ > interval_local_end (tz, --i))
         return -1;
     }
 
-  else if (time > interval_local_end (tz, i))
+  else if (time_ > interval_local_end (tz, i))
     {
-      if (time < interval_local_start (tz, ++i))
+      if (time_ < interval_local_start (tz, ++i))
         return -1;
     }
 
   else if (interval_isdst (tz, i) != type)
     {
-      if (i && time <= interval_local_end (tz, i - 1))
+      if (i && time_ <= interval_local_end (tz, i - 1))
         i--;
 
-      else if (i < tz->timecnt && time >= interval_local_start (tz, i + 1))
+      else if (i < tz->timecnt && time_ >= interval_local_start (tz, i + 1))
         i++;
     }
 
