@@ -28,7 +28,6 @@
 #include <glib/gthread.h>
 #include <glib/ghash.h>
 
-#include "galias.h"
 
 /* < private >
  * GVariantTypeInfo:
@@ -868,6 +867,3 @@ g_variant_type_info_assert_no_infos (void)
 {
   g_assert (g_variant_type_info_table == NULL);
 }
-
-#define __G_VARIANT_TYPE_INFO_C__
-#include "galiasdef.c"
