@@ -414,6 +414,12 @@ test_comments (void)
   check_name ("key comment", comment, key_comment, 0);
   g_free (comment);
 
+  g_key_file_remove_comment (keyfile, "group1", "key2", &error);
+  check_no_error (&error);
+  comment = g_key_file_get_comment (keyfile, "group1", "key2", &error);
+  check_no_error (&error);
+  g_assert (comment == NULL);
+
   comment = g_key_file_get_comment (keyfile, "group2", NULL, &error);
   check_no_error (&error);
   check_name ("group comment", comment, group_comment, 0);
@@ -1278,9 +1284,112 @@ test_int64 (void)
   g_key_file_free (file);
 }
 
+static void
+test_load (void)
+{
+  GKeyFile *file;
+  GError *error;
+  gboolean bools[2] = { TRUE, FALSE };
+
+  file = g_key_file_new ();
+  error = NULL;
+  g_assert (g_key_file_load_from_data_dirs (file, "keyfiletest.ini", NULL, 0, &error));
+  g_assert_no_error (error);
+
+  g_key_file_set_locale_string (file, "test", "key4", "de", "Vierter Schlüssel");
+  g_key_file_set_boolean_list (file, "test", "key5", bools, 2);
+  g_key_file_set_integer (file, "test", "key6", 22);
+  g_key_file_set_double (file, "test", "key7", 2.5);
+  g_key_file_set_comment (file, "test", "key7", "some float", NULL);
+  g_key_file_set_comment (file, "test", NULL, "the test group", NULL);
+  g_key_file_set_comment (file, NULL, NULL, "top comment", NULL);
+
+  g_key_file_free (file);
+}
+
+static void
+test_non_utf8 (void)
+{
+  GKeyFile *file;
+  static const char data[] =
+"[group]\n"
+"a=\230\230\230\n"
+"b=a;b;\230\230\230;\n"
+"c=a\\\n";
+  gboolean ok;
+  GError *error;
+  gchar *s;
+  gchar **l;
+
+  file = g_key_file_new ();
+
+  ok = g_key_file_load_from_data (file, data, strlen (data), 0, NULL);
+  g_assert (ok);
+
+  error = NULL;
+  s = g_key_file_get_string (file, "group", "a", &error);
+  g_assert_error (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_UNKNOWN_ENCODING);
+  g_assert (s == NULL);
+
+  g_clear_error (&error);
+  l = g_key_file_get_string_list (file, "group", "b", NULL, &error);
+  g_assert_error (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_UNKNOWN_ENCODING);
+  g_assert (l == NULL);
+
+  g_clear_error (&error);
+  l = g_key_file_get_string_list (file, "group", "c", NULL, &error);
+  g_assert_error (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
+  g_assert (l == NULL);
+
+  g_clear_error (&error);
+
+  g_key_file_free (file);
+}
+
+#ifndef SRCDIR
+#define SRCDIR "."
+#endif
+
+static void
+test_page_boundary (void)
+{
+  GKeyFile *file;
+  GError *error;
+  gint i;
+
+#define GROUP "main_section"
+#define KEY_PREFIX "fill_abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvw_"
+#define FIRST_KEY 10
+#define LAST_KEY 99
+#define VALUE 92
+
+  g_test_bug ("640695");
+
+  file = g_key_file_new ();
+
+  error = NULL;
+  g_key_file_load_from_file (file, SRCDIR "/pages.ini", G_KEY_FILE_NONE, &error);
+  g_assert_no_error (error);
+
+  for (i = FIRST_KEY; i <= LAST_KEY; i++)
+    {
+      gchar *key;
+      gint val;
+
+      key = g_strdup_printf (KEY_PREFIX "%d", i);
+      val = g_key_file_get_integer (file, GROUP, key, &error);
+      g_free (key);
+      g_assert_no_error (error);
+      g_assert_cmpint (val, ==, VALUE);
+    }
+
+}
+
 int
 main (int argc, char *argv[])
 {
+  g_setenv ("XDG_DATA_HOME", SRCDIR, TRUE);
+
   g_test_init (&argc, &argv, NULL);
   g_test_bug_base ("http://bugzilla.gnome.org/");
 
@@ -1304,6 +1413,9 @@ main (int argc, char *argv[])
   g_test_add_func ("/keyfile/key-names", test_key_names);
   g_test_add_func ("/keyfile/reload", test_reload_idempotency);
   g_test_add_func ("/keyfile/int64", test_int64);
-  
+  g_test_add_func ("/keyfile/load", test_load);
+  g_test_add_func ("/keyfile/non-utf8", test_non_utf8);
+  g_test_add_func ("/keyfile/page-boundary", test_page_boundary);
+
   return g_test_run ();
 }
